/*
 *  Copyright (C) 2004,2005,2006 Nils Faerber <nils.faerber@kernelconcepts.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 */

#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>

#include "uinput.h"
#include "dev_uinput.h"

#define BUS_RS232               0x13

#ifdef N770
#include <time.h>
#include <glib.h>
#define DBUS_API_SUBJECT_TO_CHANGE
#include <dbus/dbus.h>
DBusConnection *conn = NULL;
DBusMessage *msg_dispon = NULL;
DBusMessage *msg_dispblpause = NULL;
DBusError dbus_error;
dbus_bool_t dresult;
#define MCE_SERVICE "com.nokia.mce"
#define MCE_REQUEST_PATH "/com/nokia/mce/request"
#define MCE_REQUEST_IF "com.nokia.mce.request"
#define MCE_PREVENT_BLANK_REQ "req_display_blanking_pause"
#define MCE_DISPLAY_ON_REQ "req_display_state_on"
#endif


int dev_uinput_init(void)
{
struct uinput_dev dev;
int fd, aux;

	fd = open("/dev/uinput", O_RDWR);
	if (fd < 0)
		fd = open("/dev/misc/uinput", O_RDWR);
	if (fd < 0)
		fd = open("/dev/devfs/misc/uinput", O_RDWR);
	if (fd < 0)
		fd = open("/dev/input/uinput", O_RDWR);
	if (fd < 0) {
		perror("failed to open uinput device");
		return -1;
	}

	memset(&dev, 0, sizeof(dev));
	strncpy(dev.name, "SerKBD", UINPUT_MAX_NAME_SIZE);
	dev.idbus = BUS_RS232;
	dev.idvendor = 0x00;
	dev.idproduct = 0x00;
	dev.idversion = 0x00;

	if (write(fd, &dev, sizeof(dev)) < 0) {
		fprintf(stderr,"failed to write uinputdev: %s\n", strerror(errno));
		close(fd);
		return -1;
	}

	if (ioctl(fd, UI_SET_EVBIT, EV_KEY) != 0) {
		close(fd);
		return -1;
	}
	if (ioctl(fd, UI_SET_EVBIT, EV_REP) != 0) {
		close(fd);
		return -1;
	}
	for (aux = KEY_RESERVED; aux <= KEY_UNKNOWN; aux++)
		if (ioctl(fd, UI_SET_KEYBIT, aux) != 0) {
			close(fd);
			return -1;
		}
	if (ioctl(fd, UI_DEV_CREATE) != 0) {
		close(fd);
		return -1;
	}
#ifdef N770
	dbus_error_init(&dbus_error);
	conn = dbus_bus_get (DBUS_BUS_SYSTEM, &dbus_error);
	if (!conn)
		fprintf(stderr,"cannot initialise DBUS system connection\n");
	msg_dispon = dbus_message_new_method_call(MCE_SERVICE,
                                   MCE_REQUEST_PATH,
                                   MCE_REQUEST_IF,
                                   MCE_DISPLAY_ON_REQ);
	msg_dispblpause = dbus_message_new_method_call(MCE_SERVICE,
                                   MCE_REQUEST_PATH,
                                   MCE_REQUEST_IF,
                                   MCE_PREVENT_BLANK_REQ);
#endif
return fd;
}

int dev_uinput_key(int fd, unsigned short code, int pressed)
{
struct uinput_event event;
#ifdef N770
DBusMessage *msg=NULL;
static time_t last_msg_time = 0;
time_t curr_time = time(NULL);

	if (conn && ((curr_time-last_msg_time) > 59)) {
		/* unblank display as it may be already off */
		msg = msg_dispon;
	} else if (conn && ((curr_time-last_msg_time) > 5)) {
		/* prevent blanking each 5 seconds */
		msg = msg_dispblpause;
	}
	if (msg) {
    		// ok we should send something
    		// flush if message is still in queue
    		// doesn't work, private data //if (msg->locked) 
    		dbus_connection_flush(conn); //flush queue, just to be sure last message was sent
    		dresult = dbus_connection_send (conn, msg, NULL);
    		last_msg_time = curr_time;
    		if (!dresult)
			fprintf(stderr,"cannot send DBUS message\n");
		// we reuse message so no unref // dbus_message_unref(msg);
		// flush could block now, do later // dbus_connection_flush(conn);
	}
#endif
	memset(&event, 0, sizeof(event));
	event.type = EV_KEY;
	event.code = code;
	event.value = pressed; // (0 release, 1 press?)

return (write(fd, &event, sizeof(event)));
}

void dev_uinput_close(int fd)
{
	ioctl(fd, UI_DEV_DESTROY);
	close(fd);
#ifdef N770
	if (msg_dispon)
		dbus_message_unref(msg_dispon);
	if (msg_dispblpause)
		dbus_message_unref(msg_dispblpause);
#endif
}
